/*
   this module supports Mikroelektronika "GPS Click - Breakout Board for u-blox LEA-6s"(c)
   the code does lend itself to add further NMEA messages if you desire
   
   documents used:
   
   1. GPS.G6-HW-09007 - R13 - hardware integration manual
       "https://www.u-blox.com/sites/default/files/products/documents/LEA-NEO-MAX-6_HIM_(UBX-14054794).pdf"
   
   2. UBX-14044797 - R11 - LEA-6 - u-blox 6 GPS Modules Data Sheet
       "https://www.u-blox.com/sites/default/files/products/documents/LEA-6_DataSheet_(UBX-14044797).pdf"
       
   3. GPS.G6-SW-10018-F - u-blox 6 Receiver Description (including Protocol Specification)
       "https://www.u-blox.com/sites/default/files/products/documents/u-blox6_ReceiverDescrProtSpec_%28GPS.G6-SW-10018%29_Public.pdf?utm_source=en%2Fimages%2Fdownloads%2FProduct_Docs%2Fu-blox6_ReceiverDescriptionProtocolSpec_%28GPS.G6-SW-10018%29.pdf"
   
   
   Library Dependencies: 
   
      C_String"
      C_Type
      UART
      

   !! ATTENTION !!
   
      "unsigned short gps_get_NMEA_frame (char *gps_frame_id)" is typically
      all which needs to be called. BUT NOTE, interrupts are enabled at
      the beginning of this function and disabled on the way out. EXCEPT
      the 'global interrupt enable bit"
      ALSO, on the way out gpsVAR.hours is changed from gps-utc-time to
      local time according to gpsVAR.timeZone
   
   !! one call from the main program which has to be called is: !!
   
   1. gps_initialise(11);
     
      -- void gps_initialise (signed short int _timezone)
         {
           gps_rxPtr= &gps_rxBuf;                                   // init ptr to gps_rxBuf
           gpsPtr   = &gpsVAR;                                      // init ptr to gpsVAR
           memset    (&gpsVAR,0x00,sizeof(gpsVAR));                 // clear gpsVAR
           gps_reset_vars ();
           ANSELC.B7 = 0;                                           // set RC7 as digital I/O
           ANSELC.B6 = 0;                                           // set RC6 as digital I/O
           UART1_Init (9600);                                       // init uart to 9600 bps
		   gpsPTR->timeZone = _timezone;                            // set timezone
         }
   

      
   timezone only support hourly increments from (-11 to 14 any more exotic timezones
   such as 15min - 20min - 30min or 45min increments well, knock
   yourself out.
   
   this code contains interrupt service routine and in my final version
   there are only 4 wires connected.
   
   3.3V
   GNG
   Tx -> RC7
   TimePulse (used for something special)
   
   I never used 'mikroBUS'(c) for this gps unit on easyPic7 (c)
   P1845k22 @ 8Mhz
   
   enjoy - and please no question, complaint or otherwise!
      
*/
#define GPGLL

#ifdef GPGLL
char                 *frameID    = "$GPGLL";          // msg identity to search for
const unsigned short  GLL_NS_pos     = 18;            // GPGLL pos of N/S indicator
const unsigned short  GLL_EW_pos     = 32;            // GPGLL pos of E/W indicator
const unsigned short  GLL_valid_pos  = 44;            // GPGLL pos of data invalid or receiver error ('A'->Valid)
const unsigned short  GLL_mode_pos   = 46;            // GPGLL pos of Position Mode ('A' = good. 'V' error)
const unsigned short  GLL_crc_pos    = 48;            // GPGLL pos of first crc char
const unsigned short  GLL_maxRxBuf   = 56;            // max number of chars in rxBuf[array]
const unsigned short  GLL_crc_start  =  1;            // crc is between '$' and '*'
const unsigned short  GLL_crc_end    = GLL_crc_pos-1; // crc is between '$' and '*'

const unsigned short  GLL_hour_pos   = 34;            // start pos of 'hours'     in rxBuf
const unsigned short  GLL_minute_pos = 36;            // start pos of 'minutes'   in rxBuf
const unsigned short  GLL_second_pos = 38;            // start pos of 'seconds'   in rxBuf
const unsigned short  GLL_lat_pos    =  7;            // start pos of 'latitude'  in rxBuf
const unsigned short  GLL_long_pos   = 20;            // start pos of 'longitude' in rxBuf

#endif

const unsigned int    max_gps_delay_ticks = 60000; // equals 30sec

typedef struct gps_STRUCT
{
              double  latitude;
                char  NS_indicator;
              double  longitude;
                char  EW_indicator;
      unsigned short  hours;
      unsigned short  minutes;
      unsigned short  seconds;
    signed short int  timeZone;
}gpsREC;

        gpsREC     gpsVAR;
        gpsREC    *gpsPtr;
          char    *gps_rxPtr;
unsigned short     gps_chrCount;
          char     gps_rxBuf[GLL_maxRxBuf];
unsigned short     gps_ready;
  unsigned int     gps_delay_ticks;  // max of 500uS x 65535 = ~32 seconds
  
  
extern void gps_initialise (signed short int _timezone);

extern unsigned short gps_get_NMEA_frame (char *gps_frame_id);



